package cz.autocont.iszr.demo;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;

import cz.autocont.iszr.demo.e100.DemoCallE100;
import cz.autocont.iszr.demo.e256.DemoCallE256;
import cz.autocont.iszr.demo.e278.DemoCallE278;
import cz.autocont.iszr.demo.e99.DemoCallE99;
import cz.autocont.iszr.demo.trust.KeyStorePrinter;
import cz.autocont.iszr.demo.trust.KeyStorePrinterFactory;

/**
 * Implementace referencniho agenta.
 * Agent reaguje na prikazy ze standartniho vstupu.
 * <div style="font:italic">Projekt ISZR - (c) AutoCont CZ a.s. All rights reserved</div>
 * 
 * @author Vladimir Pach
 * @since 0.0.1
 */
public class ReferentialAgent implements Demo
{

    /**
     * Algoritmus referencniho agenta.<br/>
     * Algoritmus nejdrive zavola {@link #callIszrRobCtiPodleUdaju2()} a {@link #callIszrRosCtiIco()}.
     * Vysledek techto volani bude zobrazen na standartni vystup aplikace.
     * Pak prejde na do interaktivni casti algoritmu:
     * <ul>
     * <li>V kazdem cyklu zobrazi napovedu.</li>
     * <li>Stisk klavesy U + enter - ukonceni aplikace</li>
     * <li>Stisk klavesy F + enter - zavola {@link #callIszrAsyncVypisFronty()}</li>
     * <li>Stisk klavesy P + enter - zavola {@link #callIszrAsyncOdpovedZFronty()}</li>
     * </ul>
     * 
     * @throws DemoException Chyba při zpracování agenta
     */
    @Override
    public void execute() throws DemoException
    {
        // Vytisteni keystore a truststore
        printKeyStore();

        // volani webove sluzby
        System.out.println(Execute.config.getProperty("print.section"));
        callIszrRobCtiPodleUdaju2();

        // volani webove sluzby
        System.out.println(Execute.config.getProperty("print.section"));
        callIszrRosCtiIco();

        // Interaktivni cast dema
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in), 256);
        label:
        while (true)
        {
            System.out.println(Execute.config.getProperty("print.section"));
            System.out.println("Ukoncit aplikaci (u), provest vypis fronty (f), precist vysledek E256 (p)?");

            // Precteni prikazu ze vstupu
            String command = "";
            try
            {
                command = in.readLine();
            }
            catch (IOException e)
            {
                throw new DemoException("Can't read user command form standart input.", e);
            }

            // Nepodarilo se nic precist ze vstupu
            if (command == null)
            {
                throw new IllegalStateException("Can't read anything from standart input");
            }

            // Vyhodnoceni prikazu
            command = command.toUpperCase();
            switch (command)
            {
                case "U":
                    // Ukonceni aplikace
                    System.out.println("Ukonceni aplikace");
                    break label;
                case "F":
                    // volani webove sluzby
                    try
                    {
                        callIszrAsyncVypisFronty();
                    } catch (DemoException e)
                    {
                        System.out.println("Error by calling web service.");
                        e.printStackTrace();
                    }

                    break;
                case "P":
                    // volani webove sluzby
                    try
                    {
                        callIszrAsyncOdpovedZFronty();
                    } catch (DemoException e)
                    {
                        System.out.println("Error by calling web service.");
                        e.printStackTrace();
                    }
                    break;
                default:
                    // neznamy prikaz
                    System.out.println("Neznámý příkaz: " + command);
                    break;
            }
        }
    }

    /**
     * Na standardni vystup vytisken obsah truststore a keystore.
     *
     * @throws DemoException Chyba při výpisu key store
     */
    protected void printKeyStore() throws DemoException
    {
        // parametry truststore
        String trustStore = System.getProperty("javax.net.ssl.trustStore");
        String trustStoreType = System.getProperty("javax.net.ssl.trustStoreType");
        String trustStorePassword = System.getProperty("javax.net.ssl.trustStorePassword");
        // parametry keystore
        String keyStore = System.getProperty("javax.net.ssl.keyStore");
        String keyStoreType = System.getProperty("javax.net.ssl.keyStoreType");
        String keyStorePassword = System.getProperty("javax.net.ssl.keyStorePassword");

        // print trust store
        if (trustStore != null)
        {
            // truststore je definovany
            System.out.println(Execute.config.getProperty("print.section"));
            // tisk
            KeyStorePrinterFactory factory = KeyStorePrinterFactory.getInstance(trustStoreType);
            KeyStorePrinter printer = factory.createPrinter(new File(trustStore), trustStorePassword);
            printer.printKeyStore();
        }
        // print key store
        if (keyStore != null)
        {
            // keystore je definovany
            System.out.println(Execute.config.getProperty("print.section"));
            // tisk
            KeyStorePrinterFactory factory = KeyStorePrinterFactory.getInstance(keyStoreType);
            KeyStorePrinter printer = factory.createPrinter(new File(keyStore), keyStorePassword);
            printer.printKeyStore();
        }
    }

    /**
     * Zavola {@link DemoCallE278}.
     *
     * @throws DemoException Chyba při volání služby E289
     */
    protected void callIszrRobCtiPodleUdaju2() throws DemoException
    {
        Demo demo = new DemoCallE278();
        demo.execute();
    }

    /**
     * Zavola {@link DemoCallE256}.
     *
     * @throws DemoException Chyba při volání služby E256
     */
    protected void callIszrRosCtiIco() throws DemoException
    {
        Demo demo = new DemoCallE256();
        demo.execute();
    }

    /**
     * Zavola {@link DemoCallE99}.
     * 
     * @throws DemoException Chyba při volání služby E99
     */
    protected void callIszrAsyncVypisFronty() throws DemoException
    {
        Demo demo = new DemoCallE99();
        demo.execute();
    }

    /**
     * Zavola {@link DemoCallE100}.
     *
     * @throws DemoException Chyba při volání služby E100
     */
    protected void callIszrAsyncOdpovedZFronty() throws DemoException
    {
        Demo demo = new DemoCallE100();
        demo.execute();
    }
}
