package cz.autocont.iszr.demo.trust;

import java.io.File;
import java.io.FileInputStream;
import java.security.Key;
import java.security.KeyStore;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.util.Collections;

import cz.autocont.iszr.demo.DemoException;

/**
 * Printer pro uloziste typu JKS, kde jsou certifikaty typu X509.
 * <div style="font:italic">Projekt ISZR - (c) AutoCont CZ a.s. All rights reserved</div>
 * 
 * @author Vladimir Pach
 * @since 0.0.1
 */
public class KeyStorePrinter
{
    /**
     * Soubor uloziste. Nikdy neni null.
     */
    private File store;

    /**
     * Heslo do uloziste. Pokud uloziste neni zaheslovane, pak je null.
     * Predpoklada se, ze heslo do uloziste a hesla k jednotlivim ptrivatnim klicum, jsou stejna.
     */
    private String password;

    /**
     * @param store {@link #store}
     * @param password {@link #password}
     */
    public KeyStorePrinter(File store, String password)
    {
        super();
        this.store = store;
        this.password = password;
    }

    /**
     * Provede tisk informaci o key store na standardni vystup.
     * 
     * @throws DemoException Pokud pri tisku dojde k chybe.
     */
    public void printKeyStore() throws DemoException
    {
        try
        {
            // Ziska instanci poskytovatele
            KeyStore ks = KeyStore.getInstance(KeyStore.getDefaultType());
            // Soubor key store
            FileInputStream fis = new FileInputStream(getStore());

            // Nahrati key store do pameti
            System.out.println("Nahravam key store: " + getStore().getPath());
            ks.load(fis, getPassword().toCharArray());
            // Tisk informaci o jednotlivych polozkach key store
            for (String alias : Collections.list(ks.aliases()))
            {
                System.out.println(alias);
                if (ks.isKeyEntry(alias))
                {
                    // Tisk privatniho klice
                    System.out.println(alias + " je vstupni bod pro klic v ulozisti");
                    Key key = ks.getKey(alias, getPassword().toCharArray());
                    System.out.println("Privatni klic pro " + alias + " je " + key);
                    Certificate certs[] = ks.getCertificateChain(alias);
                    // Identita vlastnika klice
                    if (certs[0] instanceof X509Certificate)
                    {
                        X509Certificate x509 = (X509Certificate) certs[0];
                        System.out.println(alias + " je ve skutecnosti " + x509.getSubjectDN());
                    }
                    // retezec certifikatu, ktere klic verifikovali
                    for (int i=1 ; i< certs.length; i++)
                    {
                        if (certs[i] instanceof X509Certificate)
                        {
                            X509Certificate x509 = (X509Certificate) certs[i];
                            System.out.println(alias + " byl verifikovan autoritou " + x509.getSubjectDN());
                        }
                    }
                }
                else if (ks.isCertificateEntry(alias))
                {
                    // Tisk certifikatu
                    Certificate cert = ks.getCertificate(alias);
                    if (cert instanceof X509Certificate)
                    {
                        X509Certificate x509 = (X509Certificate) cert;
                        System.out.println(alias + " je ve skutecnosti " + x509.getSubjectDN());
                        System.out.println(alias + " byl verifikovan autoritou " + x509.getIssuerDN());
                    }
                }
            }
        }
        catch (Exception e)
        {
            throw new DemoException("Can't print keyStore: " + getStore().getPath(), e);
        }

    }

    /**
     * @return the store
     */
    protected File getStore()
    {
        return store;
    }

    /**
     * @return the password
     */
    protected String getPassword()
    {
        return password;
    }

}
